@@ -15,7 +15,9 @@
 # limitations under the License.
 
 import logging
-from typing import Dict
+from typing import Dict, List
+
+import pandas as pd
 from google.cloud.dialogflowcx_v3beta1 import services
 from google.cloud.dialogflowcx_v3beta1 import types
 from google.protobuf import field_mask_pb2
@@ -52,6 +54,135 @@ class EntityTypes(ScrapiBase):
         self.entity_id = entity_id
         self.agent_id = agent_id
 
+
+    @staticmethod
+    def entity_type_proto_to_dataframe(
+        obj: types.EntityType, mode: str = "basic"
+    ):
+        """Entity Types to dataframe
+
+        Args:
+          obj, EntityType protobuf object
+          mode: (Optional) basic returns display_name, value of entity type and
+            it's synonyms.
+          advanced returns entity types and excluded phrases in a comprehensive
+            format.
+        """
+        if mode == "basic":
+            main_df = pd.DataFrame()
+
+            entity_type_dict = {}
+            entity_type_dict["display_name"] = obj.display_name
+            for entity in obj.entities:
+                entity_type_dict["entity_value"] = entity.value
+                for synonym in entity.synonyms:
+                    entity_type_dict["synonyms"] = synonym
+                    main_df = pd.concat(
+                        [main_df, entity_type_dict], ignore_index=True)
+
+            return main_df
+
+        elif mode == "advanced":
+
+            main_df = pd.DataFrame()
+            excl_phrases_df = pd.DataFrame()
+
+            excl_phrases_dict = {
+                "entity_type_id": obj.name,
+                "display_name": obj.display_name,
+            }
+            entity_type_dict = {
+                "entity_type_id": obj.name,
+                "display_name": obj.display_name,
+                "kind": obj.kind.name,
+                "auto_expansion_mode": obj.auto_expansion_mode,
+                "fuzzy_extraction": obj.enable_fuzzy_extraction,
+                "redact": obj.redact,
+            }
+            for entity in obj.entities:
+                entity_type_dict["entity_value"] = entity.value
+                for synonym in entity.synonyms:
+                    entity_type_dict["synonyms"] = synonym
+                    main_df = pd.concat(
+                        [main_df, entity_type_dict], ignore_index=True)
+
+            for excluded_phrase in obj.excluded_phrases:
+                excl_phrases_dict["excluded_phrase"] = excluded_phrase.value
+                excl_phrases_df = pd.concat(
+                    [excl_phrases_df, excl_phrases_dict], ignore_index=True)
+
+            return {
+                "entity_types": main_df, "excluded_phrases": excl_phrases_df
+            }
+
+        else:
+            raise ValueError("Mode types: [basic, advanced]")
+
+    def entity_types_to_df(
+        self,
+        agent_id: str = None,
+        mode: str = "basic",
+        entity_type_subset: List[str] = None) -> pd.DataFrame:
+        """Extracts all Intents and Training Phrases into a Pandas DataFrame.
+
+        Args:
+          agent_id, agent to pull list of intents
+          mode: (Optional) basic returns display_name, value of entity type and
+            it's synonyms.
+          advanced returns entity types and excluded phrases in a comprehensive
+            format.
+          entity_type_subset: (Optional) A list of entities to pull
+            If it's None, grab all the entity_types
+        """
+
+        if not agent_id:
+            agent_id = self.agent_id
+
+        entity_types = self.list_entity_types(agent_id)
+        if mode == "basic":
+            main_df = pd.DataFrame()
+            for obj in entity_types:
+                if (entity_type_subset and
+                        obj.display_name not in entity_type_subset):
+                    continue
+
+                single_entity_df = self.entity_type_proto_to_dataframe(
+                    obj, mode=mode
+                )
+                main_df = pd.concat([main_df, single_entity_df])
+            main_df = main_df.sort_values(
+                ["display_name", "entity_value"])
+            return main_df
+
+        elif mode == "advanced":
+            main_df = pd.DataFrame()
+            excl_phrases_df = pd.DataFrame()
+            for obj in entity_types:
+                if (entity_type_subset and
+                        obj.display_name not in entity_type_subset):
+                    continue
+                single_entity_dict = self.entity_type_proto_to_dataframe(
+                    obj, mode=mode
+                )
+                main_df = pd.concat(
+                    [main_df, single_entity_dict["entity_types"]])
+                excl_phrases_df = pd.concat(
+                    [excl_phrases_df, single_entity_dict["excluded_phrases"]])
+            type_map = {
+                "auto_expansion_mode": bool,
+                "fuzzy_extraction": bool,
+                "redact": bool
+            }
+            main_df = main_df.astype(type_map)
+
+            return {
+                "entity_types": main_df, "excluded_phrases": excl_phrases_df
+            }
+
+        else:
+            raise ValueError("Mode types: [basic, advanced]")
+
+
     def get_entities_map(self, agent_id: str = None, reverse=False):
         """Exports Agent Entityt Names and UUIDs into a user friendly dict.
 
