@@ -15,12 +15,11 @@
 # limitations under the License.
 
 import logging
-from typing import Dict
+from typing import Dict, List
 from google.cloud.dialogflowcx_v3beta1 import services
 from google.cloud.dialogflowcx_v3beta1 import types
 from google.protobuf import field_mask_pb2
-
-from dfcx_scrapi.core.scrapi_base import ScrapiBase
+from dfcx_scrapi.core import scrapi_base
 
 # logging config
 logging.basicConfig(
@@ -30,7 +29,7 @@ logging.basicConfig(
 )
 
 
-class Flows(ScrapiBase):
+class Flows(scrapi_base.ScrapiBase):
     """Core Class for CX Flow Resource functions."""
 
     def __init__(
@@ -54,7 +53,11 @@ class Flows(ScrapiBase):
 
         self.agent_id = agent_id
 
-    def get_flows_map(self, agent_id: str = None, reverse=False):
+    def get_flows_map(
+        self,
+        agent_id: str,
+        reverse=False
+    ):
         """Exports Agent Flow Names and UUIDs into a user friendly dict.
 
         Args:
@@ -65,8 +68,6 @@ class Flows(ScrapiBase):
             - flows_dict, Dictionary containing flow UUIDs as keys and
                 flow.display_name as values
         """
-        if not agent_id:
-            agent_id = self.agent_id
 
         if reverse:
             flows_dict = {
@@ -82,7 +83,7 @@ class Flows(ScrapiBase):
 
         return flows_dict
 
-    def train_flow(self, flow_id: str = None):
+    def train_flow(self, flow_id: str) -> str:
         """trains the specified flow.
 
         Args:
@@ -95,8 +96,6 @@ class Flows(ScrapiBase):
             check the status of the export using
               dfcx_scrapi.core.operations->get_lro()
         """
-        if not flow_id:
-            flow_id = self.flow_id
 
         request = types.flow.TrainFlowRequest()
         request.name = flow_id
@@ -109,7 +108,10 @@ class Flows(ScrapiBase):
 
         return response
 
-    def list_flows(self, agent_id: str = None):
+    def list_flows(
+        self,
+        agent_id: str
+    ) -> List[types.Flow]:
         """Get a List of all Flows in the current Agent.
 
         Args:
@@ -117,10 +119,8 @@ class Flows(ScrapiBase):
             projects/<PROJECT ID>/locations/<LOCATION ID>/agents/<AGENT ID>
 
         Returns:
-          flows, a List of Flow objects
+          List of Flow objects
         """
-        if not agent_id:
-            agent_id = self.agent_id
 
         request = types.flow.ListFlowsRequest()
         request.parent = agent_id
@@ -135,19 +135,47 @@ class Flows(ScrapiBase):
         for page in response.pages:
             for flow in page.flows:
                 flows.append(flow)
-
         return flows
 
-    def get_flow(self, flow_id: str = None):
+    def get_flow_by_display_name(
+        self,
+        display_name: str,
+        agent_id: str
+    ) -> types.Flow:
+        """Get a single CX Flow object based on its display name.
+
+        Args:
+          display_name: The display name of the desired Flow.
+          agent_id: CX Agent ID in which the flow exists.
+
+        Returns:
+          a single CX Flow object"""
+
+        flows_map = self.get_flows_map(
+            agent_id = agent_id,
+            reverse = True
+        )
+
+        if display_name in flows_map:
+            flow_id = flows_map[display_name]
+        else:
+            raise ValueError(
+                f"Flow \"{display_name}\" " \
+                f"does not exist in the specified agent."
+            )
+
+        flow = self.get_flow(flow_id=flow_id)
+
+        return flow
+
+    def get_flow(self, flow_id: str) -> types.Flow:
         """Get a single CX Flow object.
 
         Args:
-          flow_id, CX Flow ID in the proper format
+          flow_id: CX Flow ID in the proper format
 
         Returns:
-          response, a single CX Flow object"""
-        if not flow_id:
-            flow_id = self.flow_id
+          response: a single CX Flow object"""
 
         client_options = self._set_region(flow_id)
         client = services.flows.FlowsClient(
@@ -158,8 +186,11 @@ class Flows(ScrapiBase):
         return response
 
     def update_flow(
-        self, flow_id: str = None, obj: types.Flow = None, **kwargs
-    ):
+        self,
+        flow_id: str,
+        obj: types.Flow = None,
+        **kwargs
+    ) -> types.Flow:
         """Update a single specific CX Flow object.
 
         Args:
@@ -174,8 +205,6 @@ class Flows(ScrapiBase):
             flow = obj
             flow.name = flow_id
         else:
-            if not flow_id:
-                flow_id = self.flow_id
             flow = self.get_flow(flow_id)
 
         # set flow attributes to args
@@ -192,7 +221,7 @@ class Flows(ScrapiBase):
 
         return response
 
-    def update_nlu_settings(self, flow_id, **kwargs):
+    def update_nlu_settings(self, flow_id:str, **kwargs):
         """updates flow to new NLU setting.
         Args:
             flow_id: flow id to update nlu settings for.
@@ -213,7 +242,8 @@ class Flows(ScrapiBase):
         self,
         flow_id: str,
         gcs_path: str,
-        ref_flows: bool = True) -> Dict[str, str]:
+        ref_flows: bool = True
+    ) -> Dict[str, str]:
         """Exports DFCX Flow(s) into GCS bucket.
 
         Args:
@@ -231,9 +261,7 @@ class Flows(ScrapiBase):
         request = types.flow.ExportFlowRequest()
         request.name = flow_id
         request.include_referenced_flows = ref_flows
-
-        if gcs_path:
-            request.flow_uri = gcs_path
+        request.flow_uri = gcs_path
 
         client_options = self._set_region(flow_id)
         client = services.flows.FlowsClient(
@@ -243,17 +271,48 @@ class Flows(ScrapiBase):
 
         return response.result()
 
+    def export_flow_inline(
+        self,
+        flow_id: str,
+        ref_flows: bool = True
+    ) -> bytes:
+        """Export a Flow, returning uncompressed raw byte content for flow.
+
+        Args:
+          flow_id, the formatted CX Flow ID to export
+          ref_flows, Whether to export flows referenced by the specified flow.
+
+        Returns:
+          bytes representing the content of the flow.
+        """
+        request = types.flow.ExportFlowRequest()
+        request.name = flow_id
+        request.include_referenced_flows = ref_flows
+
+
+        client_options = self._set_region(flow_id)
+        client = services.flows.FlowsClient(
+            credentials=self.creds, client_options=client_options
+        )
+        response = client.export_flow(request)
+
+        return (response.result()).flow_content
+
     def import_flow(
         self,
         agent_id: str,
-        gcs_path: str,
-        import_option: str = "KEEP") -> Dict[str, str]:
-        """Imports a DFCX Flow from GCS bucket to CX Agent.
+        gcs_path: str = None,
+        flow_content: bytes = None,
+        import_option: str = "KEEP"
+    ) -> Dict[str, str]:
+        """Imports a DFCX Flow to CX Agent. Flow can be imported from a
+        GCS bucket or from raw bytes.
 
         Args:
           agent_id, the CX Agent ID to import the flow into.
-          gcs_path, The `Google Cloud Storage URI to import flow from. The
-            format of this URI must be ``gs://<bucket-name>/<object-name>``.
+          gcs_path, the `Google Cloud Storage URI to import flow from.
+            the format of this URI must be ``gs://<bucket-name>/<object-name>``.
+          flow_content, uncompressed raw byte content for flow.
           import_option, one of 'FALLBACK' or 'KEEP'. Defaults to 'KEEP'
 
         Returns:
@@ -261,9 +320,21 @@ class Flows(ScrapiBase):
             the newly imported Flow. Otherwise, it will return the
             corresponding error.
         """
+
+        if gcs_path and flow_content:
+            raise ValueError(
+                "gcs_path or flow_content required (But not both!)."
+            )
+
+        if not gcs_path and not flow_content:
+            raise ValueError(
+                "gcs_path or flow_content required (But not both!)."
+            )
+
         request = types.flow.ImportFlowRequest()
         request.parent = agent_id
         request.flow_uri = gcs_path
+        request.flow_content = flow_content
         request.import_option = import_option
 
         client_options = self._set_region(agent_id)
@@ -275,7 +346,11 @@ class Flows(ScrapiBase):
 
         return response
 
-    def delete_flow(self, flow_id: str = None, force: bool = False) -> None:
+    def delete_flow(
+        self,
+        flow_id: str,
+        force: bool = False
+    ):
         """Deletes a single CX Flow Object resources.
 
         Args:
@@ -283,8 +358,6 @@ class Flows(ScrapiBase):
           force: False means a flow will not be deleted if a route to the flow
             exists, True means the flow will be deleted and all
         """
-        if not flow_id:
-            flow_id = self.flow_id
 
         request = types.DeleteFlowRequest()
         request.name = flow_id
