@@ -20,10 +20,10 @@ import json
 from typing import Dict
 from google.oauth2 import service_account
 from google.auth.transport.requests import Request
-
 from google.protobuf import json_format  # type: ignore
 
-
+from proto.marshal.collections import repeated
+from proto.marshal.collections import maps
 class ScrapiBase:
     """Core Class for managing Auth and other shared functions."""
 
@@ -38,7 +38,7 @@ class ScrapiBase:
         creds_dict: Dict[str,str] = None,
         creds: service_account.Credentials =None,
         scope=False,
-        agent_path=None,
+        agent_id=None,
     ):
 
         self.scopes = ScrapiBase.global_scopes
@@ -65,8 +65,8 @@ class ScrapiBase:
             self.creds = None
             self.token = None
 
-        if agent_path:
-            self.agent_path = agent_path
+        if agent_id:
+            self.agent_id = agent_id
 
     @staticmethod
     def _set_region(item_id):
@@ -117,3 +117,28 @@ class ScrapiBase:
         """convert to json so we can get at the object"""
         blob = ScrapiBase.cx_object_to_dict(msg)
         return blob.get("payload")  # deref for nesting
+
+    def recurse_proto_repeated_composite(self, repeated_object):
+        repeated_list = []
+        for item in repeated_object:
+            if isinstance(item, repeated.RepeatedComposite):
+                item = self.recurse_proto_repeated_composite(item)
+                repeated_list.append(item)
+            elif isinstance(item, maps.MapComposite):
+                item = self.recurse_proto_marshal_to_dict(item)
+                repeated_list.append(item)
+            else:
+                repeated_list.append(item)
+
+        return repeated_list
+
+    def recurse_proto_marshal_to_dict(self, marshal_object):
+        new_dict = {}
+        for k,v in marshal_object.items():
+            if isinstance(v, maps.MapComposite):
+                v = self.recurse_proto_marshal_to_dict(v)
+            elif isinstance(v, repeated.RepeatedComposite):
+                v = self.recurse_proto_repeated_composite(v)
+            new_dict[k] = v
+
+        return new_dict
